////////////////////////////////////////////////////////////////////////////////
// JavaScript library to load any .NET assembly/type and execute against its 
// public methods and properties
////////////////////////////////////////////////////////////////////////////////

function GadgetBuilder()
{
    // ProgID of the Gadget Adapter for use in creating an ActiveX object instance
    var progID = "Kens.Gadget.TypeLoader";
    
    // File name of the Gadget Interop DLL
    var assemblyName = "Kens.Gadget.TypeLoader.dll";
    
    // Gadget Adapter class GUID
    var guid = "{FFFB4D5D-19EA-4ce3-BE12-B4F51D72FE50}";
    
    // Gadget Adapter assembly location (under the gadget's root directory)
    var assemblyStore = "\\bin\\";
    
    // Instance of the ActiveX gadget adapter object
    this._builder;  
    
    // Method pointers
    this.Initialize = Initialize;
    this.LoadType = LoadType;
    this.LoadTypeWithParams = LoadTypeWithParams;
    this.UnloadType = UnloadType;
    this.AddConstructorParam = AddConstructorParam;
    this.InteropRegistered = InteropRegistered;
    this.UnregisterGadgetInterop = UnregisterGadgetInterop;
    this.GetActiveXObject = GetActiveXObject;
    
    ////////////////////////////////////////////////////////////////////////////////
    // Initializes this class by creating an instance of the Gadget Adapter.  If the
    // Gadget Adapter assembly is not yet retistered, the corresponding registry
    // values are added to register the Gadget Adapter.  (i.e. runtime registration)
    ////////////////////////////////////////////////////////////////////////////////
    function Initialize()
    {
        // Check if the gadget adapter needs to be registered
        if(InteropRegistered() == false)
        {
            // Register the adapter since an instance couldn't be created.
            RegisterGadgetInterop();
        }
        
        if(InteropRegistered() == false)
        {
            WriteLine("Type loader registration failed");
        }
        else
        {
            WriteLine("Type loader registration successful");
        }
        
        // Load an instance of the Gadget Adapter as an ActiveX object
        _builder = GetActiveXObject();
    }
    
    ////////////////////////////////////////////////////////////////////////////////
    // Load the specified .NET type from the given assembly
    ////////////////////////////////////////////////////////////////////////////////
    function LoadType(assemblyPath, classType)
    {
        try
        {
            return _builder.LoadType(assemblyPath, classType);
        }
        catch(e)
        {
            WriteLine(e.message);
            WriteLine(e.description);
            
            return null;
        }
    }
    
    ////////////////////////////////////////////////////////////////////////////////
    // Load the specified type from the given assembly.  The "preserve" argument provides 
    // control over clearing the constructor arguments after object creating
    ////////////////////////////////////////////////////////////////////////////////
    function LoadTypeWithParams(assemblyPath, classType, preserve)
    {
        try
        {
            return _builder.LoadTypeWithParams(assemblyPath, classType, preserve);
        }
        catch(e)
        {
            WriteLine(e.message);
            WriteLine(e.description);
            
            return null;
        }
    }
    
    ////////////////////////////////////////////////////////////////////////////////
    // Call the object's Dispose method
    ////////////////////////////////////////////////////////////////////////////////
    function UnloadType(typeToUnload)
    {
        try
        {
            _builder.UnloadType(typeToUnload);
        }
        catch(e)
        {
            WriteLine(e.message);
            WriteLine(e.description);
        }
    }
    
    ////////////////////////////////////////////////////////////////////////////////
    // Add a parameter to the list of objects needed to call the class' constructor
    ////////////////////////////////////////////////////////////////////////////////
    function AddConstructorParam(param)
    {
        _builder.AddConstructorParam(param);
    }
    
    ////////////////////////////////////////////////////////////////////////////////
    // Add the Kens.Gadget.TypeLoader dll to the registry so it can be used by COM and
    // created in javascript as an ActiveX object
    ////////////////////////////////////////////////////////////////////////////////
    function RegisterGadgetInterop()
    {
        WriteLine("Registering type loader");
        
        try
        {
            // Full path to the Kens.Gadget.TypeLoader.dll assembly
            var fullPath = System.Gadget.path + assemblyStore;
            var asmPath = fullPath + assemblyName;
            
            // Register the interop assembly under the Current User registry key
            RegAsmInstall("HKCU", progID, "Kens.Gadget.TypeLoader", guid,
                            "Kens.Gadget.TypeLoader, Version=1.0.0.0, Culture=neutral, PublicKeyToken=36ea59d17931cf0f",
                            "1.0.0.0", asmPath);
        }
        catch(e)
        {
            WriteLine(e.message);
            WriteLine(e.description);
        }    
    }
        
    ////////////////////////////////////////////////////////////////////////////////
    // Code to register a .NET type for COM interop
    ////////////////////////////////////////////////////////////////////////////////
    function RegAsmInstall(root, progId, cls, clsid, assembly, version, codebase) 
    {
        var wshShell;
        wshShell = new ActiveXObject("WScript.Shell"); 

        wshShell.RegWrite(root + "\\Software\\Classes\\" + progId + "\\", cls);
        wshShell.RegWrite(root + "\\Software\\Classes\\" + progId + "\\CLSID\\", clsid);       
        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\", cls); 

        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\InprocServer32\\", "mscoree.dll");
        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\InprocServer32\\ThreadingModel", "Both");
        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\InprocServer32\\Class", cls);
        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\InprocServer32\\Assembly", assembly);
        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\InprocServer32\\RuntimeVersion", "v2.0.50727");
        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\InprocServer32\\CodeBase", codebase); 

        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\InprocServer32\\" + version + "\\Class", cls);
        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\InprocServer32\\" + version + "\\Assembly", assembly);
        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\InprocServer32\\" + version + "\\RuntimeVersion", "v2.0.50727");
        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\InprocServer32\\" + version + "\\CodeBase", codebase); 

        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\ProgId\\", progId); 

        wshShell.RegWrite(root + "\\Software\\Classes\\CLSID\\" + clsid + "\\Implemented Categories\\{62C8FE65-4EBB-45E7-B440-6E39B2CDBF29}\\", "");
    } 
    
    ////////////////////////////////////////////////////////////////////////////////
    // Remove the Kens.Gadget.TypeLoader dll from the registry and the GAC
    ////////////////////////////////////////////////////////////////////////////////
    function UnregisterGadgetInterop()
    {
        try
        {
            var fullPath = System.Gadget.path + assemblyStore;
            var asmPath = fullPath + assemblyName;
            
            RegAsmUninstall("HKCU", progID, "Kens.Gadget.TypeLoader", guid,
                            "Kens.Gadget.TypeLoader, Version=1.0.0.0, Culture=neutral, PublicKeyToken=36ea59d17931cf0f",
                            "1.0.0.0", asmPath); 
        }
        catch(e)
        {
            WriteLine(e.message);
            WriteLine(e.description);
        }
    }
    
    ////////////////////////////////////////////////////////////////////////////////
    // Unregister a component
    ////////////////////////////////////////////////////////////////////////////////
    function RegAsmUninstall(root, progId, clsid) 
    {
        var wshShell;
        wshShell = new ActiveXObject("WScript.Shell"); 

        wshShell.RegDelete(root + "\\Software\\Classes\\" + progId);
        wshShell.RegDelete(root + "\\Software\\Classes\\CLSID\\" + clsid);
    } 

    ////////////////////////////////////////////////////////////////////////////////
    // Returns true if a Gadget Adapter ActiveX object is successfully created,
    // otherwise returns false.
    ////////////////////////////////////////////////////////////////////////////////
    function InteropRegistered()
    {
        try
        {
            var proxy = GetActiveXObject();
            proxy = null;
            
            return true;
        }
        catch(e)
        {
            WriteLine(e.message);
            WriteLine(e.description);
            
            return false;
        }
    }
    
    ////////////////////////////////////////////////////////////////////////////////
    // Attempts to create and return an instance of the Gadget Adapter ActiveX object.
    ////////////////////////////////////////////////////////////////////////////////
    function GetActiveXObject()
    {
        try
        {
            return new ActiveXObject(progID);
        }
        catch(e)
        {
            WriteLine(e.message);
            WriteLine(e.description);
            
            throw e;
        }
        
        return null;
    }
    
    ////////////////////////////////////////////////////////////////////////////////
    // Replace all occurrences of a character
    ////////////////////////////////////////////////////////////////////////////////
    function ReplaceAll(input, oldValue, newValue)
    { 
        var tempInput = input; 
        var i = tempInput.indexOf(oldValue); 
        
        while(i > -1)
        { 
            tempInput = tempInput.replace(oldValue, newValue); 
            i = tempInput.indexOf(oldValue); 
        } 
        
        return tempInput; 
    }
}