// Settings
var address;
var userID;
var password;
var frequency;

var largeWindowDocked;
var largeWindowUndocked;

var backgroundColor;

var largeWindowHeight;
var valuesHistoryWindowHeight;
var timerHistoryWindowHeight;
var alarmTimer;
var statusWebsite;
var historyLogSize;
var playSound;

// Other globals
var latency = 1000;
var timeout;
var lastUpdated = "Never";

var showingValueFlyout;

// Instance of the .NET type loader
var builder = new GadgetBuilder();

// Initialize the adapter to call the .NET assemblies
builder.Initialize();

// Instance of AC3StatusReader
var ac3StatusReader;

function Start()
{
    WriteLine("Start()");

    ReadSettings();

	CheckForUpdate();

    System.Gadget.settingsUI = "Settings.html";
    System.Gadget.onSettingsClosed = SettingsClosed;
    
    System.Gadget.onDock = SetDockedSize;
    System.Gadget.onUndock = SetUndockedSize;
    
    // Run once.  I cannot call the method directly since
    //  the gadget is not initialized and won't resolve the 
    //  document.elements correctly.
    timeout = setTimeout('SendRequest()', latency);
}

function Stop()
{
    WriteLine("Stop()");

    if (timeout != null)
    {
        clearTimeout(timeout);
        timeout = null;
    }
    
    if (refreshUpdateTimer != null)
    {
        clearTimeout(refreshUpdateTimer);
        refreshUpdateTimer = null;
    }
    
    if (ac3StatusReader != null &&
        builder != null)
    {
        builder.UnloadType(ac3StatusReader);
        ac3StatusReader = null;
    }
    
    if (versionResults != null &&
        builder != null)
    {
        builder.UnloadType(versionResults);
        versionResults = null;
    }

    if (builder != null)
    {
        builder.UnregisterGadgetInterop()
    }
}

function ReadSettings()
{
    WriteLine("ReadSettings()");

    if (System.Gadget.Settings.read("SettingExist") == true)
    {
        address = System.Gadget.Settings.read("address");
        
        userID = System.Gadget.Settings.read("userName");
        password = System.Gadget.Settings.read("password");
        
        frequency = System.Gadget.Settings.read("frequency");
        
        alarmTimer = System.Gadget.Settings.read("alarmTimer");
        
        statusWebsite = System.Gadget.Settings.read("statusWebsite");
        
        historyLogSize = System.Gadget.Settings.read("historyLogSize");
        
        playSound = System.Gadget.Settings.read("playSound");
        
        largeWindowDocked = System.Gadget.Settings.read("largeWindowDocked");
        largeWindowUndocked = System.Gadget.Settings.read("largeWindowUndocked");
        largeWindowHeight = System.Gadget.Settings.read("largeWindowHeight");
        
        valuesHistoryWindowHeight = System.Gadget.Settings.read("valuesHistoryWindowHeight");
        timerHistoryWindowHeight = System.Gadget.Settings.read("timerHistoryWindowHeight");
        
        backgroundColor = System.Gadget.Settings.read("backgroundColor");
    }
    else
    {
        address = "192.168.1.50:80";
        
        userID = "admin";
        password = "1234";
        
        frequency = 60000;
        
        alarmTimer = "ALM";
        
        playSound = true;
        
        statusWebsite = "";
        
        historyLogSize = 100;
        
        largeWindowDocked = false;
        largeWindowUndocked = true;
        largeWindowHeight = 375;
        
        valuesHistoryWindowHeight = 500;
        timerHistoryWindowHeight = 500;
        
        backgroundColor = "black";
    }
}

function SendRequest()
{
    WriteLine("SendRequest()");

    UpdateSizes();

    // Exit if no credentials exist
    if(address == "" ||
        userID == "" ||
        password == "")
    {
        WriteLine("Missing address, user or password, nothing to do");
        
        return;
    }
    
    WriteLine("HTTP Get executing");
    
    if (ac3StatusReader == null)
    {
//        // Build the params for the ActiveX class
//        builder.AddConstructorParam(address);
//        builder.AddConstructorParam(userID);
//        builder.AddConstructorParam(password);

        // Load the GmailReader.dll assembly and create an instance of the GmailReader.GmailClient type
        ac3StatusReader = builder.LoadType(System.Gadget.path + "\\bin\\ac3StatusReader.dll", "AC3StatusReader.StatusReader");
        
        if (ac3StatusReader != null)
        {
            // TODO: figure out why code only works with empty ctor, parameters don't work????
            ac3StatusReader.Address = address;
            ac3StatusReader.Username = userID;
            ac3StatusReader.Password = password;
            ac3StatusReader.AlarmTimer = alarmTimer;
            ac3StatusReader.HistorySize = historyLogSize;
        }
        else
        {
            var link2 = document.getElementById('ac3Status');
            
            if (link2 != null)
            {
                link2.innerText = "Initialization Failed";
            }
            
            return;
        }
    }
    
    try
    {
        // Start a 'Get'
        ac3StatusReader.ReadStatus();
    }
    finally
    {
        // Check every 1 second to see if AC3 has replied
        timeout = setTimeout('ProcessRequest()', latency);
    }
}

function ProcessRequest()
{
    WriteLine("ProcessRequest()");

    if (ac3StatusReader.ReplyFound == true)
    {
        WriteLine("Reply found, updating status");
        
        try
        {
            var link3 = document.getElementById('ac3Status');
            
            if (link3 != null)
            {
                if (ac3StatusReader.AlarmTimerOn == true)
                {
                    link3.style.color = "red";
                    
                    if (playSound == true)
                    {
                        System.Sound.playSound(System.Gadget.path + "\\bin\\alarm.wav");
                    }
                }
                else
                {
                    link3.style.color = "green";
                }
                
                link3.innerHTML = ac3StatusReader.Result;
            }
            
            WriteLine(ac3StatusReader.Result);
        
            var link4 = document.getElementById('deviceState');
            
            WriteLine(ac3StatusReader.Timers);

            if (link4 != null)
            {
                link4.innerHTML = ac3StatusReader.Timers;
            }
        
            GetDateString();
            
            WriteLine("Last Update: " + lastUpdated);
            
            var mailLink = document.getElementById('lastUpdated');
            
            if (mailLink != null)
            {
                mailLink.innerText = "Last Update: " + lastUpdated;
            }
        }
        catch (e)
        {
            WriteLine(e.message);
            WriteLine(e.description);
        }
        finally
        {
            // Wait for timeout, then get the next update
            timeout = setTimeout('SendRequest()', frequency);
        }
    }
    else
    {
        // Give the AC3 another second to return the reply
        WriteLine("Reply not found, wait a moment");

        timeout = setTimeout('ProcessRequest()', latency);
    }
}

function GetDateString()
{
    var d = new Date();

    var iMin = d.getMinutes();
    var iSec = d.getSeconds();
    
    lastUpdated = d.getHours();

    if (iMin < 10)
        lastUpdated = lastUpdated + ":0" + iMin;
    else
        lastUpdated = lastUpdated + ":" + iMin;

    if (iSec < 10)
        lastUpdated = lastUpdated + ":0" + iSec;
    else
        lastUpdated = lastUpdated + ":" + iSec;
}

function SetDockedSize()
{
    WriteLine("SetDockedSize()");

    UpdateSizes();
}

function SetUndockedSize()
{
    WriteLine("SetUndockedSize()");

    UpdateSizes();
}

function UpdateSizes()
{
    WriteLine("UpdateSizes()");

    document.getElementById("body").style.backgroundColor = backgroundColor;

    var showLargeWindow = false;

    if(System.Gadget.docked == true) 
    {
        showLargeWindow = largeWindowDocked;
    } 
    else if (System.Gadget.docked == false)
    {
        showLargeWindow = largeWindowUndocked;
    }
    
    if(showLargeWindow == true)
    {
        SetLargeParameters();
    }
    else
    {
        SetSmallParameters();
    }
}

function SetSmallParameters()
{
    WriteLine("SetSmallParameters()");

    document.body.style.height = "210px";

    document.getElementById("container").style.height = "210";
    document.getElementById("deviceState").style.height = "82px";
}

function SetLargeParameters()
{
    WriteLine("SetLargeParameters()");

    var bodyAndContainerHeight = largeWindowHeight.toString(10) + "px";
    
    document.body.style.height = bodyAndContainerHeight;
    document.getElementById("container").style.height = bodyAndContainerHeight;
    
    var deviceStatusHeight = largeWindowHeight - 128;
    
    document.getElementById("deviceState").style.height = deviceStatusHeight.toString() + "px";
}

function Refresh()
{
    WriteLine("Refresh()");

    if (timeout != null)
    {
        clearInterval(timeout);
    }
    
    ReadSettings();
    
    if (ac3StatusReader != null)
    {
        ac3StatusReader.Address = address;
        ac3StatusReader.Username = userID;
        ac3StatusReader.Password = password;
        ac3StatusReader.AlarmTimer = alarmTimer;
        ac3StatusReader.HistorySize = historyLogSize;
    }

    var link3 = document.getElementById('ac3Status');

    if (link3 != null)
    {
        link3.innerHTML = "";
    }
    
    var link4 = document.getElementById('deviceState');

    if (link4 != null)
    {
        link4.innerHTML = "";
    }
    
    var mailLink = document.getElementById('lastUpdated');
    
    if (mailLink != null)
    {
        mailLink.innerText = "Last Update: Never";
    }

    timeout = setTimeout('SendRequest()', latency);
}

function SettingsClosed()
{
    WriteLine("SettingsClosed()");

    if (System.Gadget.Settings.read("SettingExist") == true) 
    {
        Refresh();
	}
}

function OpenAC3Website()
{
    WriteLine("OpenAC3Website()");

    var connection
    
    if (statusWebsite == null ||
        statusWebsite == "")
    {
        connection = "http://" + address + "/cgi-bin/status.cgi";
    }
    else
    {
        connection = statusWebsite;
    }
    
    System.Shell.execute(connection);
}
